
-- Note:  Use Euphoria v4.0.5, (not v4.1, because of user input and output)

-- James Jonathan Cook

-- Description: "a living record"

-- it should only record commands that it executed.

include std/console.e
include std/get.e
include std/io.e
include std/pretty.e
include std/text.e
include std/types.e

global constant kill_stop_watch = 1

-- with trace
-- trace(1)

include aprompt.e
include rpn.e

constant recordfile = "record.log"

-- sequence record
-- record = read_lines(recordfile)

integer recordfn
recordfn = open(recordfile, "a")


-- for performance:
constant max_characters = 1000

constant filename = "answers.dat"

procedure save_answers()
	integer fn
	fn = open(filename, "w")
	write_lines(fn, stack)
	write_lines(fn, answers)
	close(fn)
end procedure

procedure load_answers()
	integer fn
	object lines
	sequence st
	fn = open(filename, "r")
	if fn = -1 then
		save_answers()
		lines = read_lines(filename)
	else
		lines = read_lines(fn)
		close(fn)
	end if
	-- answers need to be parsed and recorded.
	for i = 1 to length(lines) do
		st = lines[i]
		answers = answers & parse(st, whitespace, "\"")
		--puts(recordfn, st & "\n")
	end for
end procedure

integer key
sequence answer

function get_answer(sequence s)
	sequence st
	
	st = prompt_string(s)
	if length(st) > max_characters then
		st = st[1..max_characters]
	end if
	
	printf(1, "Did you say \'%s\'?\n[yn] ", {st})
	while 1 do
		key = wait_key()
		if key = 'y' or key = 'Y' then
			answers = answers & parse(st, whitespace, "\"")
			--puts(recordfn, st & "\n")
			--record = append(record, st)
			exit
		elsif key = 'n' or key = 'N' then
			exit
		end if
	end while
	puts(1, "\n")
	--answer = upper(st)
	if length(answers) then
		answer = upper(answers[1])
	else
		answer = {}
	end if
	return st
end function

function main(sequence cmdln)
	sequence st, tokens, val
	object token, ans
	integer found, f
	found = 0
	
	load_answers()
	
	puts(1, "Type \'q\' and then enter to quit.  (Dont use CTRL-C.)\nTry typing in \'list commands\'\n")
	
	start_stopwatch()
	while stop_stopwatch() < 60 do -- give it 60 seconds to think
	--while 1 do
		if input_tokens_left() = 0 then
			key = 0
			while key != 'y' and key != 'Y' do
				st = get_answer("\nWhat should I do? ")
-- 				if find(answer, end_session) then
-- 					--key = -2
-- 					--exit
-- 					exit 2
-- 				end if
			end while
			--if key = -2 then
			--	exit
			--end if
			start_stopwatch()
		end if
		token = get_next_token()
		
		-- token is an operator or an operand
		
		answer = upper(token)
		--token = lower(token)
		--puts(recordfn, token & "\n")
		switch answer do
		case "EXIT", "Q" then
			puts(recordfn, answer & "\n")
			tokens = {} -- deallocate to free memory
			val = {}
			ans = {}
			exit
		case "PROMPT" then
			-- takes no arguments.
			key = 0
			while key != 'y' and key != 'Y' do
				st = get_answer("")
-- 				if find(answer, end_session) then
-- 					--key = -2
-- 					--exit
-- 					exit 2
-- 				end if
			end while
			--if key = -2 then
			--	exit
			--end if
			start_stopwatch()
		case "DISPLAY" then
			if length(stack) < 1 then
				puts(1, "Cannot display, no items in stack.\n")
			else
				tokens = pop(1)
				ans = tokens[1]
				if t_print(ans) then
					puts(1, ans)
					token = ans & " " & token
				else
					pretty_print(1, ans, {2})
					token = sprint(ans) & " " & token
				end if
			end if
		case "PEEK" then
			if length(stack) < 1 then
				puts(1, "Cannot display, no items in stack.\n")
			else
				pretty_print(1, stack, {2})
-- 				ans = stack[$]
-- 				if atom(ans) then
-- 					ans = sprintf("%g", {ans})
-- 				end if
-- 				puts(1, ans)
			end if
		
		-- Simple math functions:
		case "+", "ADD" then
			if length(stack) < 2 then
				puts(1, "Not enough items in stack.\n")
				continue
			end if
			tokens = pop(2)
			if sequence(tokens[1]) then
				val = value(tokens[1])
				tokens[1] = val[2]
			end if
			if sequence(tokens[2]) then
				val = value(tokens[2])
				tokens[2] = val[2]
			end if
			ans = tokens[1] + tokens[2] -- do the addition
			push_onto_stack(ans)
		case "-", "SUBTRACT" then
			if length(stack) < 2 then
				puts(1, "Not enough items in stack.\n")
				continue
			end if
			tokens = pop(2)
			if sequence(tokens[1]) then
				val = value(tokens[1])
				tokens[1] = val[2]
			end if
			if sequence(tokens[2]) then
				val = value(tokens[2])
				tokens[2] = val[2]
			end if
			ans = tokens[1] - tokens[2] -- do the subtraction
			push_onto_stack(ans)
		case "*", "MULTIPLY" then
			if length(stack) < 2 then
				puts(1, "Not enough items in stack.\n")
				continue
			end if
			tokens = pop(2)
			if sequence(tokens[1]) then
				val = value(tokens[1])
				tokens[1] = val[2]
			end if
			if sequence(tokens[2]) then
				val = value(tokens[2])
				tokens[2] = val[2]
			end if
			ans = tokens[1] * tokens[2] -- do the multiplication
			push_onto_stack(ans)
		case "/", "DIVIDE" then
			if length(stack) < 2 then
				puts(1, "Not enough items in stack.\n")
				continue
			end if
			tokens = pop(2)
			if sequence(tokens[1]) then
				val = value(tokens[1])
				tokens[1] = val[2]
			end if
			if sequence(tokens[2]) then
				val = value(tokens[2])
				tokens[2] = val[2]
			end if
			ans = tokens[1] / tokens[2] -- do the division
			push_onto_stack(ans)
		
		-- Default case:
		case else
			-- token is a value
			push_onto_stack(token)
			answer = token
			--continue
			-- do something
		end switch
		puts(recordfn, answer & "\n")
		tokens = {} -- deallocate to free memory
		val = {}
		ans = {}
		
		-- do something here.
		
		
		
		
		
		
		
-- 		-- simulate thinking:
-- 		found = 0
-- 		for i = length(answers) - 1 to 1 by -1 do
-- 			if equal(st, answers[i]) then
-- 				found = i
-- 				exit
-- 			end if
-- 		end for
-- 		if found then
-- 			printf(1, "This is line \'%d\'.\n", {length(answers)})
-- 			printf(1, "You last said \'%s\' on line \'%d\'.\n", {st, found})
-- 			-- if you want to delete that line, you must delete it manually from the data file.
-- 		end if
		
		
		
		
	end while
	
	save_answers()
	printf(1, "\nOperating on %d number of answers.\n", {length(stack) + length(answers)})
	
	return 0
end function
abort(main(command_line()))
--end of file.
